function New-WC
{
  <#
  .SYNOPSIS
  Returns a new webclient object
  .DESCRIPTION
  Returns a new webclient object, 
  with the header "Content-Type", "text/xml; charset=utf-8" added
  set to use the default windows credentials
  .EXAMPLE
  $webclient = New-WC
  #>
    param
    (
        [switch] $IgnoreSSL,
        [switch] $CookieAware,
        [bool] $UseDefaultCredentials = $true,
        $headers
    )

    if($IgnoreSSL) {[System.Net.ServicePointManager]::ServerCertificateValidationCallback = {$true}}

    if($CookieAware)
    {
$CookieAwareWebClient = @"
public class CookieAwareWebClient : System.Net.WebClient
{
  public CookieAwareWebClient()
  {
    System.Net.ServicePointManager.Expect100Continue = true;
  }

  private System.Net.CookieContainer m_container = new System.Net.CookieContainer();
  protected override System.Net.WebRequest GetWebRequest(System.Uri address)
  {
    System.Net.WebRequest request = base.GetWebRequest(address);
    if (request is System.Net.HttpWebRequest)
    {
      (request as System.Net.HttpWebRequest).CookieContainer = m_container;
    }
    return request;
  }
}
"@
        Add-Type -TypeDefinition $CookieAwareWebClient
        $webClient = New-Object CookieAwareWebClient
        foreach($header in $headers)
        {
          $webClient.Headers.add($header.Name, $header.Value)
        }
        $webClient.UseDefaultCredentials = $true
    }
    else
    {
        $webClient = New-Object System.Net.WebClient
        foreach($header in $headers)
        {
          $webClient.Headers.add($header.Name, $header.Value)
        }
    }

    if($UseDefaultCredentials) {$webClient.UseDefaultCredentials = $true}
    return $webClient
}

function Send-Web_Post
{
    param (
        [string] $URL, 
        [string] $postData, 
        [object] $webClient
    )
    Write-Debug $URL
    Write-Debug $postData

    $res = $webClient.UploadString($URL, $postData)
    if($res) { return $res }
    else { return $false }
}

function Send-ShoreTelDirectorLogin
{
    param 
    (
        [string] $shoretelServer,
        [bool] $windowsAuth,
        [string] $user,
        [string] $pass
    )

    if($shoretelServer -eq ""){Throw "You must provide a ShoreTel server ip or hostname" }
    if($windowsAuth -eq $false -and ($user -eq "" -or $pass -eq "")){ Throw "You must provide a username or password if not using windows authentication" }

    $session = @{}
    $session.Server = $shoretelServer
    $session.SessionType = "Director"
    $session.Authenticated = $false
    $session.SequenceId = 1
    $session.webClient = New-WC -windowsAuth $windowsAuth -CookieAware

    if($windowsAuth)
    {
        $queryString = "?doingSSO=1"
        $response = Get-Web -url ("http://" + $shoretelServer + "/shorewaredirector/validate.asp" + $queryString) -webClient $session.webClient
    }
    else
    {   
        $session.webClient.Headers.Add("Content-Type", "application/x-www-form-urlencoded")
        $md5Pass = ConvertTo-MD5 -string $pass
        $postData = "hpassword=" + $md5Pass
        $postData += "&login=" + $user
        $postData += "&password="
        $postData += "&SUBMIT1=Login"
        $response = Send-Web_Post -url ("http://" + $shoretelServer + "/shorewaredirector/validate.asp") -postData $postData -webClient $session.webClient
    }
    if($response.StartsWith("<script> location.href='mainframe.asp'</script>"))
    {
        $session.Authenticated = $true
    }
    return $session
}

function ConvertTo-MD5($string) {
    $md5 = new-object -TypeName System.Security.Cryptography.MD5CryptoServiceProvider
    $utf8 = new-object -TypeName System.Text.UTF8Encoding
    $hash = [System.BitConverter]::ToString($md5.ComputeHash($utf8.GetBytes($String))).Replace("-", "")
    return $hash.ToLower()
}

function Remove-ShoreTelDirectorUser
{
    param 
    (
        [string] $extension,
        [switch] $returnResults = $false,
        [object] $session
    )
    if($session.SessionType -eq "Director" -and $session.Authenticated)
    {
        $postData = "<?xml version=`"1.0`" encoding=`"utf-8`"?><soap:Envelope xmlns:soap=`"http://schemas.xmlsoap.org/soap/envelope/`"><soap:Body><Delete><param><![CDATA[" + $extension + "]]></param></Delete></soap:Body></soap:Envelope>"
        $response = Send-Web_Post -URL ("http://" + $session.Server + "/shorewaredirector/user_actions.asp") -postData $postData -webClient $session.webClient -verbose $PSBoundParameters['Verbose']
    }

    if($returnResults) 
    {
        return $response
    }
    else
    {
        Write-Debug $response
        if($response.response -gt -1) {return $true} else {return $false}
    }
}